import ccclass = cc._decorator.ccclass;
import Wx from "../../framework/wx/Wx";
import property = cc._decorator.property;

/**
 * 排行榜数据过期方式.
 */
enum DateFilter {
    NONE = 0, WEEK = 1, MONTH = 2
}

@ccclass
export class SubContextView extends cc.Component {
    private sprite: cc.Sprite = null;
    private readonly texture: cc.Texture2D = new cc.Texture2D();

    // 用于表示当前子域需要绘制哪个View.
    @property
    private viewName: string = "";

    // 约定要获取的key的名称.即 getCloudStorage(keyList:key)
    @property
    private key: string = "";

    // 用于表示子域数据根据什么key 进行排行.
    @property
    private sortKey: string = "";

    @property({
        type: cc.Enum(DateFilter),
        serializable: true,
        displayName: "过滤方式",
        tooltip: "指定排行数据是否按周/月进行过滤,仅显示本周/月的数据."
    })
    private dateFilter: DateFilter = DateFilter.NONE;

    public onLoad() {
        this.sprite = this.getComponent(cc.Sprite);
        if (!this.sprite) {
            this.sprite = this.addComponent(cc.Sprite);
        }
    }

    public start() {
    }

    public onEnable() {
        if (!Wx.isWeChatEnv()) {
            return;
        }
        if (this.viewName.length <= 0 || this.key.length <= 0) {
            console.error("SubContextView 未设置子域viewName 和key ,无法显示子域");
            return;
        }
        cc.log("SubContextView 显示子域:", this.viewName, this.key);
        const shareCanvas = Wx.getShareCanvas();
        shareCanvas.width = this.node.width;
        shareCanvas.height = this.node.height;
        Wx.sendMsgToContext("action_init_context", {view: this.viewName, bound: this.node.getContentSize()});

        this.updateView();
        this.setPage(0);
    }

    public onDisable() {
    }

    public updateView() {
        Wx.sendMsgToContext(this.viewName, {sortKey: this.sortKey, key: this.key, dateFilter: this.dateFilter});
    }

    public setPage(page: number) {
        Wx.sendMsgToContext("action_paging", {page: page, view: this.viewName});
    }

    // 向前翻页.
    public pageUp() {
        Wx.sendMsgToContext("action_paging", {offset: -1, view: this.viewName});
    }

    // 向后翻页.
    public pageDown() {
        Wx.sendMsgToContext("action_paging", {offset: 1, view: this.viewName});
    }

    public lateUpdate() {
        this.render();
    }

    private render() {
        if (!Wx.isWeChatEnv()) {
            return;
        }
        const shareCanvas = Wx.getShareCanvas();
        this.texture.initWithElement(shareCanvas);
        this.texture.handleLoadedTexture();
        if (!this.sprite.spriteFrame) {
            this.sprite.spriteFrame = new cc.SpriteFrame(this.texture);
        } else {
            this.sprite.spriteFrame.setTexture(this.texture);
        }
    }
}
